Object subclass: #IntegralApproximator
	instanceVariableNames: 'left right function weight '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Integrator'!

!IntegralApproximator methodsFor: 'initialization' stamp: 'len 9/2/1998 20:06'!
initialize
	self weight: (Function constant: 1.0)! !


!IntegralApproximator methodsFor: 'accessing' stamp: 'len 9/1/1998 04:27'!
approximation
	^ self subclassResponsibility! !

!IntegralApproximator methodsFor: 'accessing' stamp: 'len 6/26/1998 01:27'!
function: aFunction
	function _ aFunction! !

!IntegralApproximator methodsFor: 'accessing' stamp: 'len 6/26/1998 01:27'!
left: aNumber
	left _ aNumber! !

!IntegralApproximator methodsFor: 'accessing' stamp: 'len 6/26/1998 01:27'!
right: aNumber
	right _ aNumber! !

!IntegralApproximator methodsFor: 'accessing' stamp: 'len 6/26/1998 01:28'!
weight: aFunction
	weight _ aFunction! !


IntegralApproximator subclass: #GaussianIntegralApproximator
	instanceVariableNames: 'nodes weights '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Integrator'!

!GaussianIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/2/1998 20:43'!
approximation
	| nstream wstream answer |
	nstream _ ReadStream on: nodes.
	wstream _ ReadStream on: weights.
	answer _ 0.
	nodes size timesRepeat: [answer _ (function valueAt: nstream next) * wstream next + answer].
	^ answer! !

!GaussianIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/2/1998 16:41'!
nodes
	^ nodes! !

!GaussianIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/2/1998 16:41'!
nodes: aCollection
	nodes _ aCollection! !

!GaussianIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/2/1998 16:41'!
weights
	^ weights! !

!GaussianIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/2/1998 16:41'!
weights: aCollection
	weights _ aCollection! !


IntegralApproximator subclass: #MonteCarloIntegralApproximator
	instanceVariableNames: 'max min random iterations goods '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Integrator'!

!MonteCarloIntegralApproximator methodsFor: 'initialization' stamp: 'len 9/2/1998 20:06'!
initialize
	super initialize.
	iterations _ 0.
	goods _ 0.
	random _ Random new! !


!MonteCarloIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/1/1998 03:17'!
approximation
	iterations = 0 ifTrue: [self halt].
	^ right - left * (max - min) * goods / iterations! !

!MonteCarloIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/1/1998 03:01'!
max: aNumber
	max _ aNumber! !

!MonteCarloIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/1/1998 03:01'!
min: aNumber
	min _ aNumber! !


!MonteCarloIntegralApproximator methodsFor: 'computing' stamp: 'len 9/1/1998 18:51'!
iterate
	| x0 y0 y |
	iterations _ iterations + 1.
	x0 _ right - left * random next + left.
	y0 _ max - min * random next + min.
	y _ (function valueAt: x0) * (weight valueAt: x0).
	(y0 between: 0.0 and: y)
		ifTrue: [goods _ goods + 1]
		ifFalse: [(y0 between: y and: 0.0) ifTrue: [goods _ goods - 1]]! !

!MonteCarloIntegralApproximator methodsFor: 'computing' stamp: 'len 9/1/1998 03:14'!
iterate: anInteger
	anInteger timesRepeat: [self iterate]! !

!MonteCarloIntegralApproximator methodsFor: 'computing' stamp: 'len 9/1/1998 03:05'!
iterations
	^ iterations! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

MonteCarloIntegralApproximator class
	instanceVariableNames: ''!

!MonteCarloIntegralApproximator class methodsFor: 'instance creation' stamp: 'len 9/1/1998 03:18'!
new
	^ super new initialize! !


IntegralApproximator subclass: #TrapezoidalIntegralApproximator
	instanceVariableNames: 'nodes approximation increment '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Integrator'!

!TrapezoidalIntegralApproximator methodsFor: 'accessing' stamp: 'A.T. 9/17/1998 01:30'!
approximation
     nodes isNil
		ifTrue: [ nodes _ left to: right by: right - left / increment ].

	self run.
	^ approximation! !

!TrapezoidalIntegralApproximator methodsFor: 'accessing' stamp: 'A.T. 9/17/1998 01:50'!
left: aNumber
	super left: aNumber.
	nodes _ nil.
! !

!TrapezoidalIntegralApproximator methodsFor: 'accessing' stamp: 'len 9/1/1998 04:22'!
nodes: aCollection
	nodes _ aCollection! !

!TrapezoidalIntegralApproximator methodsFor: 'accessing' stamp: 'A.T. 9/17/1998 01:51'!
right: aNumber
	super right: aNumber.
	nodes _ nil.
! !

!TrapezoidalIntegralApproximator methodsFor: 'accessing' stamp: 'A.T. 9/17/1998 01:34'!
steps: anInteger    
	increment _ anInteger.
	nodes _ nil.

	! !


!TrapezoidalIntegralApproximator methodsFor: 'computing' stamp: 'len 9/1/1998 04:22'!
done
	^ approximation notNil! !

!TrapezoidalIntegralApproximator methodsFor: 'computing' stamp: 'len 9/1/1998 04:38'!
run
	| total stream x1 x2 y1 y2 |
	total _ 0.
	stream _ ReadStream on: nodes.
	x2 _ stream next.
	y2 _ (function valueAt: x2) * (weight valueAt: x2).
	[stream atEnd]
		whileFalse: [x1 _ x2.
					x2 _ stream next.
					y1 _ y2.
					y2 _ (function valueAt: x2) * (weight valueAt: x2).
					total _ x2 - x1 * (y2 + y1) + total].
	approximation _ total / 2! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

TrapezoidalIntegralApproximator class
	instanceVariableNames: ''!

!TrapezoidalIntegralApproximator class methodsFor: 'instance creation' stamp: 'A.T. 9/17/1998 00:11'!
new
	^ super new initialize! !
