Object subclass: #Complex
	instanceVariableNames: 'real imaginary '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Kernel'!
!Complex commentStamp: 'A.T. 9/17/1998 02:24' prior: 0!
My instances are complex numbers.

For instance, the complex number '2 + 3i' can be created doing:
	Complex real: 2 imaginary: 3.

Furthermore, Numbers implement the #i message. This way, '2 + 3i' can be created evaluating:
	2 + 3 i.

For some examples try the following:
	(2 + 3 i) - (2 + 3 i) conjugated
	2 + 3 i * (2 - 3 i).
	(-1 + 1 i) norm2.
	(4 + 2 i) reciprocal.
	1 i squared.
!


!Complex methodsFor: 'initialization' stamp: 'len 8/4/97 22:44'!
setReal: aNumber imaginary: anotherNumber
	real _ aNumber.
	imaginary _ anotherNumber! !


!Complex methodsFor: 'accessing' stamp: 'len 2/25/98 18:02'!
ambient
	"Answer the ambient of the receiver."

	^ self real ambient complex! !

!Complex methodsFor: 'accessing' stamp: 'len 8/4/97 22:42'!
imaginary
	"Answer the imaginary part of the receiver."

	^ imaginary! !

!Complex methodsFor: 'accessing' stamp: 'len 8/4/97 22:42'!
real
	"Answer the real part of the receiver."

	^ real! !


!Complex methodsFor: 'arithmetic' stamp: 'len 2/25/98 22:14'!
* anObject
	"Answer the product of the receiver by the argument."

	anObject isComplex
		ifFalse: [^ (anObject adaptComplex: self) * anObject adaptToComplex].
	^ self class
		real: self real * anObject real - (self imaginary * anObject imaginary)
		imaginary: self real * anObject imaginary + (self imaginary * anObject real)! !

!Complex methodsFor: 'arithmetic' stamp: 'len 2/25/98 22:14'!
+ anObject
	"Answer the sum of the receiver and the argument."

	anObject isComplex
		ifFalse: [^ (anObject adaptComplex: self) + anObject adaptToComplex].
	^ self class
		real: self real + anObject real
		imaginary: self imaginary + anObject imaginary! !

!Complex methodsFor: 'arithmetic' stamp: 'len 8/4/97 22:47'!
- anObject
	"Answer the difference between the receiver and the argument."

	^ self + anObject negated! !

!Complex methodsFor: 'arithmetic' stamp: 'len 8/4/97 22:48'!
/ anObject
	"Answer the result of dividing the receiver by the argument."

	^ self * anObject reciprocal! !

!Complex methodsFor: 'arithmetic' stamp: 'len 10/17/97 03:57'!
conjugated
	"Answer the conjugated of the receiver."

	^ self class real: self real imaginary: self imaginary negated! !

!Complex methodsFor: 'arithmetic' stamp: 'len 2/25/98 02:40'!
i
	"Answer the receiver multiplicated by i."

	^ self class real: self imaginary negated imaginary: self real! !

!Complex methodsFor: 'arithmetic' stamp: 'len 8/4/97 22:19'!
negated
	"Answer the additive inverse of the receiver."

	^ self class real: self real negated imaginary: self imaginary negated! !

!Complex methodsFor: 'arithmetic' stamp: 'len 10/17/97 03:58'!
reciprocal
	"Answer the multiplicative inverse of the receiver."

	^ self conjugated / self norm2! !


!Complex methodsFor: 'mathematical functions' stamp: 'len 3/18/98 00:13'!
abs
	"Answer absolute value of the receiver."

	^ self norm! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 9/1/97 00:33'!
arcCos
	"Answer the arc-cosinus of the receiver."

	^ (self + ((1 - self squared) sqrt * 1 i)) ln * -1 i! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 9/1/97 00:33'!
arcSin
	"Answer the arc-sinus of the receiver."

	^ ((1 - self squared) sqrt + self * 1 i) ln * -1 i! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 9/1/97 00:33'!
log: aNumber
	"Answer the log base aNumber of the receiver."

	^ self ln / aNumber ln! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 3/18/98 00:12'!
norm
	"Answer the norm of the receiver."

	^ self norm2 sqrt! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 8/4/97 22:49'!
norm2
	"Answer the square norm of the receiver."

	^ self real squared + self imaginary squared! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 9/1/97 00:40'!
raisedTo: arg
	"Answer the receiver raised to arg."

	arg isInteger ifTrue:
		["Do the special case of integer power"
		^ self raisedToInteger: arg].

	arg = 0 ifTrue: [^ 1].	"Special case of exponent=0"
	arg = 1 ifTrue: [^ self].	"Special case of exponent=1"
	^ (arg * self ln) exp		"Otherwise raise it to the power using logarithms"! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 2/25/98 03:00'!
raisedToInteger: anInteger
	"Answer the receiver raised to the power anInteger where the argument 
	must be a kind of Integer. This is a special case of raisedTo:."

	anInteger isInteger
		ifFalse: [^ self error: 'raisedToInteger: only works for integral arguments'].
	anInteger = 0 ifTrue: [^ self identity].
	anInteger = 1 ifTrue: [^ self].
	anInteger > 1 ifTrue:
		[^ (self * self raisedToInteger: anInteger // 2)
		 * (self raisedToInteger: anInteger \\ 2)].
	^ (self raisedToInteger: anInteger negated) reciprocal! !

!Complex methodsFor: 'mathematical functions'!
sin
	self notYetImplemented
! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 8/4/97 22:52'!
squared
	"Answer the square of the receiver."

	^ self class
		real: self real squared - self imaginary squared
		imaginary: self real * self imaginary * 2! !

!Complex methodsFor: 'mathematical functions' stamp: 'len 9/1/97 00:42'!
tan
	"Answer the tangent of the receiver."

	self notYetImplemented! !


!Complex methodsFor: 'comparing' stamp: 'len 2/25/98 22:18'!
= anObject
	"Answer true if the receiver equals the argument."

	anObject isNumber ifFalse: [^ false].
	anObject isComplex
		ifFalse: [^ (anObject adaptComplex: self) = anObject adaptToComplex].
	^ self real = anObject real and: [self imaginary = anObject imaginary]! !

!Complex methodsFor: 'comparing' stamp: 'len 8/4/97 22:42'!
hash
	"Answer the hash value for the receiver."

	^ self real hash + self imaginary hash! !


!Complex methodsFor: 'converting' stamp: 'len 8/4/97 22:18'!
adaptAlgebraic: anAlgebraicNumber
	"If I am involved in arithmetic with an AlgebraicNumber, do not convert me."
	^ anAlgebraicNumber asComplex! !

!Complex methodsFor: 'converting'!
adaptFloat: aFloat
	"If I am involved in arithmetic with a Float, convert the Float."
	^ aFloat asComplex! !

!Complex methodsFor: 'converting'!
adaptFraction: aFraction
	"If I am involved in arithmetic with a Fraction, convert the Fraction."
	^ aFraction asComplex! !

!Complex methodsFor: 'converting'!
adaptInteger: anInteger
	"If I am involved in arithmetic with an Integer, convert the Integer."
	^ anInteger asComplex! !

!Complex methodsFor: 'converting' stamp: 'len 2/25/98 03:25'!
adaptQuaternion: aQuaternion
	"If I am involved in arithmetic with a Quaternion, do not convert it."
	^ aQuaternion! !

!Complex methodsFor: 'converting' stamp: 'len 8/4/97 22:18'!
adaptToAlgebraic
	"If I am involved in arithmetic with an AlgebraicNumber, do not convert me."
	^ self! !

!Complex methodsFor: 'converting'!
adaptToFloat
	"If I am involved in arithmetic with a Float, do not convert me."
	^ self! !

!Complex methodsFor: 'converting'!
adaptToFraction
	"If I am involved in arithmetic with a Fraction, do not convert me."
	^ self! !

!Complex methodsFor: 'converting'!
adaptToInteger
	"If I am involved in arithmetic with an Integer, do not convert me."
	^ self! !

!Complex methodsFor: 'converting' stamp: 'len 2/25/98 03:24'!
adaptToQuaternion
	"If I am involved in arithmetic with a Quaternion, convert me."
	^ self asQuaternion! !

!Complex methodsFor: 'converting' stamp: 'len 2/25/98 03:25'!
asComplex
	^ self! !

!Complex methodsFor: 'converting' stamp: 'len 2/25/98 03:24'!
asQuaternion
	"Convert the receiver to a Quaternion."

	^ Quaternion a: self real b: self imaginary c: self real null d: self real null! !


!Complex methodsFor: 'testing' stamp: 'len 2/25/98 22:15'!
isComplex
	"Answer true if the receiver is a Complex."
	^ true! !

!Complex methodsFor: 'testing' stamp: 'len 9/1/97 00:43'!
isNull
	"Answer true if the receiver is null."

	^ self real isNull and: [self imaginary isNull]! !

!Complex methodsFor: 'testing' stamp: 'len 8/5/97 00:59'!
isNumber
	^ true! !

!Complex methodsFor: 'testing'!
isRational
	^ false! !


!Complex methodsFor: 'constants' stamp: 'len 2/25/98 03:01'!
identity
	"Answer the identity element."
	^ self class real: self real identity imaginary: self imaginary null! !

!Complex methodsFor: 'constants' stamp: 'len 2/25/98 03:01'!
null
	"Answer the null element."
	^ self class real: self real null imaginary: self imaginary null! !


!Complex methodsFor: 'printing' stamp: 'len 2/25/98 03:54'!
printOn: aStream
	"Print a representation of the receiver on the stream aStream."

	| number |
	self isNull ifTrue: [aStream print: 0. ^ self].
	self real isNull ifFalse: [aStream print: self real].
	self imaginary isNull ifTrue: [^ self].
	number _ self imaginary.
	self real isNull
		ifTrue: [number negative
					ifTrue: [aStream nextPut: $-.
							number _ number negated]]
		ifFalse: [aStream nextPutAll: (number positive
								ifTrue: [' + ']
								ifFalse: [number _ number negated. ' - '])].
	number = number identity ifFalse: [aStream print: number; space].
	aStream nextPut: $i! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Complex class
	instanceVariableNames: ''!

!Complex class methodsFor: 'instance creation'!
real: realPart imaginary: imaginaryPart
	^self new setReal: realPart imaginary: imaginaryPart! !


Object subclass: #Galois
	instanceVariableNames: 'representative modulo '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Kernel'!

!Galois methodsFor: 'accessing-private' stamp: 'len 5/6/98 00:11'!
modulo: aPolynomial
	modulo _ aPolynomial! !

!Galois methodsFor: 'accessing-private' stamp: 'len 5/6/98 00:11'!
representative: aPolynomial
	representative _ aPolynomial! !


!Galois methodsFor: 'accessing' stamp: 'len 5/6/98 00:12'!
ambient
	"Answer the ambient of the receiver."
	self notYetImplemented! !

!Galois methodsFor: 'accessing' stamp: 'len 5/6/98 00:12'!
modulo
	"Answer the modulo of the receiver."
	^ modulo! !

!Galois methodsFor: 'accessing' stamp: 'len 5/6/98 00:12'!
representative
	"Answer a representative polynomial of the receiver."
	^ representative! !


!Galois methodsFor: 'arithmetic' stamp: 'len 5/6/98 00:10'!
* anObject
	"Answer the product of the receiver with the argument."

	anObject isInteger
		ifTrue: [^ self class new: self representative * anObject \\ self modulo mod: self modulo].
	^ self class new: self representative * anObject representative \\ self modulo mod: self modulo! !

!Galois methodsFor: 'arithmetic' stamp: 'len 5/6/98 00:13'!
+ aGalois
	"Answer the sum of the receiver with the argument."

	^ self class new: self representative + aGalois representative mod: self modulo! !

!Galois methodsFor: 'arithmetic' stamp: 'len 5/6/98 00:10'!
- anObject
	"Answer the difference of the receiver with the argument."

	^ self + anObject negated! !

!Galois methodsFor: 'arithmetic' stamp: 'len 5/6/98 00:10'!
/ anObject
	"Answer the division of the receiver by the argument."

	^ self * anObject reciprocal! !

!Galois methodsFor: 'arithmetic' stamp: 'len 5/6/98 00:10'!
negated
	"Answer the additive inverse of the receiver."

	^ self isNull
		ifTrue: [self]
		ifFalse: [self class new: self modulo - self representative mod: self modulo]! !

!Galois methodsFor: 'arithmetic' stamp: 'len 5/6/98 00:15'!
reciprocal
	"Answer the multiplicative inverse of the receiver (whenever it exists)."

	| xgcd |
	self isNull ifTrue: [^ self error: 'division by zero'].
	xgcd _ (self representative xgcd: self modulo).
	^ self class new: (xgcd at: 2) \\ self modulo mod: self modulo! !

!Galois methodsFor: 'arithmetic' stamp: 'len 5/6/98 00:10'!
squared
	"Answer the square of the receiver."

	^ self class new: self representative squared \\ self modulo mod: self modulo! !


!Galois methodsFor: 'mathematical functions' stamp: 'len 5/6/98 00:10'!
raisedToInteger: anInteger
	"Answer the receiver raised to the power anInteger where
	the argument must be a kind of Integer."

	anInteger isInteger
		ifFalse: [^self error: 'raisedToInteger: only works for integral arguments'].
	anInteger = 0 ifTrue: [^ self identity].
	anInteger = 1 ifTrue: [^ self].
	anInteger > 1
		ifTrue: [^(self * self raisedToInteger: anInteger // 2)
					* (self raisedToInteger: anInteger \\ 2)].
	^ (self raisedToInteger: anInteger negated) reciprocal! !


!Galois methodsFor: 'comparing' stamp: 'len 5/6/98 00:16'!
= anObject
	"Answer true if the receiver equals the argument."

	^ self modulo = anObject modulo
		and: [self representative = anObject representative]! !

!Galois methodsFor: 'comparing' stamp: 'len 5/6/98 00:10'!
hash
	"Answer the hash value for the receiver."

	^ self representative hash! !


!Galois methodsFor: 'testing' stamp: 'len 5/6/98 00:16'!
isNull
	"Answer true if the receiver is null."

	^ self representative isNull! !


!Galois methodsFor: 'converting' stamp: 'len 5/6/98 00:17'!
adaptInteger: anInteger
	^ self identity * anInteger! !

!Galois methodsFor: 'converting' stamp: 'len 5/6/98 00:10'!
adaptToInteger
	^ self! !


!Galois methodsFor: 'constants' stamp: 'len 5/6/98 00:18'!
identity
	^ self class new: self representative identity mod: self modulo! !

!Galois methodsFor: 'constants' stamp: 'len 5/6/98 00:18'!
null
	^ self class new: self representative null mod: self modulo! !


!Galois methodsFor: 'private' stamp: 'len 5/6/98 00:10'!
errorModuloNotMatch
	self error: 'modulo not match'! !

!Galois methodsFor: 'private' stamp: 'len 5/6/98 00:18'!
reduced
	^ self class new: self representative \\ self modulo mod: self modulo! !


!Galois methodsFor: 'printing' stamp: 'len 5/6/98 00:19'!
printOn: aStream
	"Print a representation of the receiver in the stream aStream."

	self representative printOn: aStream! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Galois class
	instanceVariableNames: ''!

!Galois class methodsFor: 'instance creation' stamp: 'len 5/6/98 00:20'!
new: aPolynomial mod: anotherPolynomial
	"Answer a new instance of the receiver representing the
	residue class of aPolynomial modulo anotherPolynomial."

	^ self new modulo: anotherPolynomial; representative: aPolynomial! !


Magnitude subclass: #Infinity
	instanceVariableNames: 'sign '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Kernel'!
!Infinity commentStamp: 'A.T. 9/17/1998 02:24' prior: 0!
My instances are the real +infinity or -infinity.

For some examples try these:
	Infinity positive * Infinity negative.
	Infinity positive * 2.
	Infinity positive - 7811234871239847.
	Infinity negative / -199.
	Infinity positive reciprocal.
	Infinity positive > Infinity negative.
	Infinity negative < -19238479182374598172349871234.
	Infinity negative > 0.
	Infinity negative min: Infinity positive.

The following are examples of undeterminations (they produce an error):
	Infinity positive + Infinity negative.
	Infinity positive * 0.
	Infinity positive / Infinity positive.
	Infinity positive raisedToInteger: 0.
!


!Infinity methodsFor: 'accessing-private' stamp: 'len 10/9/97 20:16'!
sign: anInteger
	sign _ anInteger! !


!Infinity methodsFor: 'accessing' stamp: 'len 10/9/97 20:17'!
sign
	"Answer the sign of the receiver."

	^ sign! !


!Infinity methodsFor: 'arithmetic' stamp: 'len 11/14/97 14:49'!
* anObject
	"Answer the product of the receiver by the argument."

	anObject = 0 ifTrue: [^ self errorUndetermined].
	^ self class sign: self sign * anObject sign! !

!Infinity methodsFor: 'arithmetic' stamp: 'len 12/13/97 02:26'!
+ anObject
	"Answer the sum of the receiver and the argument."

	(anObject isInfinity and: [self sign ~= anObject sign])
		ifTrue: [^ self errorUndetermined].
	^ self! !

!Infinity methodsFor: 'arithmetic' stamp: 'len 11/12/97 21:30'!
- anObject
	"Answer the difference between the receiver and the argument."

	^ self + anObject negated! !

!Infinity methodsFor: 'arithmetic' stamp: 'len 10/9/97 20:24'!
/ anObject
	"Answer the division of the receiver by the argument."

	^ self * anObject reciprocal! !

!Infinity methodsFor: 'arithmetic' stamp: 'len 11/12/97 21:31'!
negated
	"Answer a copy of the receiver with the sign changed."

	^ self class sign: self sign negated! !

!Infinity methodsFor: 'arithmetic' stamp: 'len 10/17/97 04:11'!
raisedToInteger: anInteger
	"Answer the receiver raised to the integer anInteger."

	anInteger = 0 ifTrue: [^ self errorUndetermined].
	anInteger negative ifTrue: [^ 0].
	^ anInteger odd ifTrue: [self] ifFalse: [self negated]! !

!Infinity methodsFor: 'arithmetic' stamp: 'len 11/12/97 21:32'!
reciprocal
	"Answer zero. (1 / self)"

	^ 0! !

!Infinity methodsFor: 'arithmetic' stamp: 'len 10/17/97 04:09'!
squared
	"Answer the square of the receiver."

	^ self class positive! !


!Infinity methodsFor: 'comparing' stamp: 'len 12/13/97 02:26'!
< anObject
	"Answer true if the receiver is lower than the argument."

	anObject isInfinity ifTrue: [^ self sign < anObject sign].
	^ self negative! !

!Infinity methodsFor: 'comparing' stamp: 'len 11/14/97 14:35'!
= anObject
	"Answer true if the receiver equals the argument."

	^ self class == anObject class and: [self sign = anObject sign]! !

!Infinity methodsFor: 'comparing' stamp: 'len 12/13/97 02:26'!
> anObject
	"Answer true if the receiver is greater than the argument."

	anObject isInfinity ifTrue: [^ self sign > anObject sign].
	^ self positive! !

!Infinity methodsFor: 'comparing' stamp: 'len 11/14/97 14:35'!
hash
	"Answer the hash value of the receiver."

	^ self sign hash! !


!Infinity methodsFor: 'testing' stamp: 'len 12/13/97 02:25'!
isInfinity
	"Answer true if the receiver is infinity."

	^ true! !

!Infinity methodsFor: 'testing' stamp: 'len 11/14/97 14:50'!
negative
	"Answer true if the receiver is negative."

	^ self sign negative! !

!Infinity methodsFor: 'testing' stamp: 'len 11/14/97 14:50'!
positive
	"Answer true if the receiver is positive."

	^ self sign positive! !

!Infinity methodsFor: 'testing' stamp: 'len 11/14/97 14:51'!
strictlyPositive
	"Answer true if the receiver is strictly positive."

	^ self positive! !


!Infinity methodsFor: 'private' stamp: 'len 10/9/97 20:29'!
errorUndetermined
	^ self error: 'undetermined'! !


!Infinity methodsFor: 'printing' stamp: 'len 10/9/97 20:13'!
printOn: aStream
	"Print a representation of the receiver on the stream aStream."

	self negative ifTrue: [aStream nextPut: $-].
	aStream nextPutAll: 'infinity'! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Infinity class
	instanceVariableNames: ''!

!Infinity class methodsFor: 'instance creation' stamp: 'len 10/17/97 04:07'!
negative
	"Answer a new instance of the receiver representing -infinity."

	^ self sign: -1! !

!Infinity class methodsFor: 'instance creation' stamp: 'len 10/17/97 04:07'!
positive
	"Answer a new instance of the receiver representing +infinity."

	^ self sign: 1! !

!Infinity class methodsFor: 'instance creation' stamp: 'len 10/9/97 20:27'!
sign: anInteger
	"Answer a new instance of the receiver with sign anInteger."

	^ self new sign: anInteger! !


Object subclass: #IntegerMod
	instanceVariableNames: 'representative modulo '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Kernel'!
!IntegerMod commentStamp: 'A.T. 9/17/1998 02:24' prior: 0!
My instances are modular integers, i.e. elements of the quotient Z / mZ.

Modular integers are created sending the message #mod: to an Integer, with argument another Integer. For some examples try:

	(2 mod: 7) + (3 mod: 7).
	(2 mod: 7) * (3 mod: 7).
	(1231 mod: 5) reciprocal.
	(2 mod: 3) raisedToInteger: 1234081723.
	(3 mod: 6) isUnit.
	(5 mod: 6) isUnit.
!


!IntegerMod methodsFor: 'accessing-private' stamp: 'len 8/25/97 00:24'!
modulo: anInteger
	modulo _ anInteger! !

!IntegerMod methodsFor: 'accessing-private' stamp: 'len 9/28/97 00:19'!
representative: anInteger
	representative _ anInteger! !


!IntegerMod methodsFor: 'accessing' stamp: 'len 2/25/98 04:37'!
ambient
	"Answer the ambient of the receiver."

	^ AlgebraicAmbient integersMod: self modulo! !

!IntegerMod methodsFor: 'accessing' stamp: 'len 8/25/97 00:39'!
modulo
	"Answer the modulo of the receiver."

	^ modulo! !

!IntegerMod methodsFor: 'accessing' stamp: 'len 9/28/97 00:18'!
representative
	"Answer a representative integer element of the receiver between 0 and (modulo - 1)."

	^ representative! !


!IntegerMod methodsFor: 'arithmetic' stamp: 'len 9/28/97 00:36'!
* anObject
	"Answer the product of the receiver with the argument."

	anObject isInteger
		ifTrue: [^ self class new: self representative * anObject \\ self modulo mod: self modulo].
	^ self class new: self representative * anObject representative \\ self modulo mod: self modulo! !

!IntegerMod methodsFor: 'arithmetic' stamp: 'len 9/28/97 00:22'!
+ anIntegerMod
	"Answer the sum of the receiver with the argument."

	| sum |
	sum _ self representative + anIntegerMod representative.
	sum < self modulo ifFalse: [sum _ sum - self modulo].
	^ self class new: sum mod: self modulo! !

!IntegerMod methodsFor: 'arithmetic' stamp: 'len 8/25/97 00:20'!
- anObject
	"Answer the difference of the receiver with the argument."

	^ self + anObject negated! !

!IntegerMod methodsFor: 'arithmetic' stamp: 'len 8/25/97 00:20'!
/ anObject
	"Answer the division of the receiver by the argument."

	^ self * anObject reciprocal! !

!IntegerMod methodsFor: 'arithmetic' stamp: 'len 9/28/97 00:24'!
negated
	"Answer the additive inverse of the receiver."

	^ self isNull
		ifTrue: [self]
		ifFalse: [self class new: self modulo - self representative mod: self modulo]! !

!IntegerMod methodsFor: 'arithmetic' stamp: 'len 9/28/97 00:24'!
reciprocal
	"Answer the multiplicative inverse of the receiver (whenever it exists)."

	| xgcd |
	xgcd _ (self representative xgcd: self modulo).
	(xgcd at: 1) = 1 ifFalse: [^ self error: 'division by zero'].
	^ self class new: (xgcd at: 2) \\ self modulo mod: self modulo! !

!IntegerMod methodsFor: 'arithmetic' stamp: 'len 9/28/97 00:30'!
squared
	"Answer the square of the receiver."

	^ self class new: self representative squared \\ self modulo mod: self modulo! !


!IntegerMod methodsFor: 'mathematical functions' stamp: 'len 8/25/97 00:16'!
raisedToInteger: anInteger
	"Answer the receiver raised to the power anInteger where
	the argument must be a kind of Integer."

	anInteger isInteger
		ifFalse: [^self error: 'raisedToInteger: only works for integral arguments'].
	anInteger = 0 ifTrue: [^ self identity].
	anInteger = 1 ifTrue: [^ self].
	anInteger > 1
		ifTrue: [^(self * self raisedToInteger: anInteger // 2)
					* (self raisedToInteger: anInteger \\ 2)].
	^ (self raisedToInteger: anInteger negated) reciprocal! !


!IntegerMod methodsFor: 'comparing' stamp: 'len 9/28/97 00:23'!
= anObject
	"Answer true if the receiver equals the argument."

	^ self class == anObject class and: [
		self modulo = anObject modulo and: [
			self representative = anObject representative]]! !

!IntegerMod methodsFor: 'comparing' stamp: 'len 9/28/97 00:23'!
hash
	"Answer the hash value for the receiver."

	^ self representative hash! !


!IntegerMod methodsFor: 'testing' stamp: 'len 9/28/97 00:24'!
isNull
	"Answer true if the receiver is null."

	^ self representative = 0! !

!IntegerMod methodsFor: 'testing' stamp: 'len 8/25/97 00:24'!
isSquare
	"Answer true if the receiver is an square."

	self notYetImplemented! !

!IntegerMod methodsFor: 'testing' stamp: 'len 9/28/97 00:25'!
isUnit
	"Answer true if the receiver is a unit."

	^ self isNull not and: [(self representative gcd: self modulo) = 1]! !


!IntegerMod methodsFor: 'converting' stamp: 'len 9/5/97 00:28'!
adaptInteger: anInteger
	^ self class new: anInteger mod: self modulo! !

!IntegerMod methodsFor: 'converting' stamp: 'len 8/26/97 01:56'!
adaptToInteger
	^ self! !


!IntegerMod methodsFor: 'constants' stamp: 'len 9/5/97 00:29'!
identity
	^ self class new: 1 mod: self modulo! !

!IntegerMod methodsFor: 'constants' stamp: 'len 9/5/97 00:30'!
null
	^ self class new: 0 mod: self modulo! !


!IntegerMod methodsFor: 'private'!
errorModuloNotMatch
	self error: 'modulo not match'! !

!IntegerMod methodsFor: 'private' stamp: 'len 9/5/97 00:34'!
reduced
	^ self class new: self value \\ self modulo mod: self modulo! !

!IntegerMod methodsFor: 'private' stamp: 'len 12/1/97 22:42'!
value
	| value |
	value _ self representative <= (self modulo // 2)
		ifTrue: [self representative] ifFalse: [self representative - self modulo].
	^ value! !


!IntegerMod methodsFor: 'printing' stamp: 'len 4/30/98 05:05'!
printOn: aStream
	"Print a representation of the receiver in the stream aStream."

	| value |
	value _ self representative <= (self modulo // 2)
		ifTrue: [self representative] ifFalse: [self representative - self modulo].
	aStream print: value "; nextPutAll: ' mod: '; print: self modulo"! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

IntegerMod class
	instanceVariableNames: ''!

!IntegerMod class methodsFor: 'instance creation' stamp: 'len 9/28/97 00:28'!
new: anInteger mod: anotherInteger
	"Answer a new instance of the receiver representing the
	residue class of anInteger modulo anotherInteger."

	^ self new modulo: anotherInteger; representative: anInteger! !


IntegerMod subclass: #IntegerModPrime
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Kernel'!

!IntegerModPrime methodsFor: 'arithmetics' stamp: 'len 9/28/97 00:27'!
reciprocal
	"Answer the multiplicative inverse of the receiver."

	self isNull ifTrue: [^ self error: 'division by zero'].
	^ self class new: ((self representative xgcd: self modulo) at: 2) mod: self modulo! !


!IntegerModPrime methodsFor: 'testing' stamp: 'len 8/25/97 00:25'!
isUnit
	"Answer true if the receiver is a unit."

	^ self isNull not! !


Object subclass: #Quaternion
	instanceVariableNames: 'a b c d '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Kernel'!
!Quaternion commentStamp: 'A.T. 9/17/1998 02:24' prior: 0!
My instances are quaternions.
Quaternions constitute an example of non-commutative field.
!


!Quaternion methodsFor: 'accessing-private' stamp: 'len 2/25/98 02:41'!
a: aNumber b: bNumber c: cNumber d: dNumber
	a _ aNumber.
	b _ bNumber.
	c _ cNumber.
	d _ dNumber! !


!Quaternion methodsFor: 'accessing' stamp: 'len 2/25/98 02:43'!
a
	"Answer the 'a' component of the receiver."

	^ a! !

!Quaternion methodsFor: 'accessing' stamp: 'len 2/25/98 18:02'!
ambient
	"Answer the ambient of the receiver."

	^ self a ambient quaternions! !

!Quaternion methodsFor: 'accessing' stamp: 'len 2/25/98 02:43'!
b
	"Answer the 'b' component of the receiver."

	^ b! !

!Quaternion methodsFor: 'accessing' stamp: 'len 2/25/98 02:43'!
c
	"Answer the 'c' component of the receiver."

	^ c! !

!Quaternion methodsFor: 'accessing' stamp: 'len 2/25/98 02:43'!
d
	"Answer the 'd' component of the receiver."

	^ d! !


!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 02:58'!
* anObject
	"Answer the product of the receiver by the argument."

	| a0 a1 a2 a3 b0 b1 b2 b3 |
	anObject isQuaternion
		ifFalse: [^ (anObject adaptQuaternion: self) * anObject adaptToQuaternion].
	a0 _ self a. a1 _ self b. a2 _ self c. a3 _ self d.
	b0 _ anObject a. b1 _ anObject b. b2 _ anObject c. b3 _ anObject d.
	^ self class
		a: a0 * b0 - (a1 * b1) - (a2 * b2) - (a3 * b3)
		b: a0 * b1 + (a1 * b0) + (a2 * b3) - (a3 * b2)
		c: a0 * b2 + (a2 * b0) + (a3 * b1) - (a1 * b3)
		d: a0 * b3 + (a3 * b0) + (a1 * b2) - (a2 * b1)! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 02:45'!
+ anObject
	"Answer the sum of the receiver and the argument."

	anObject isQuaternion
		ifFalse: [^ (anObject adaptQuaternion: self) + anObject adaptToQuaternion].
	^ self class
		a: self a + anObject a
		b: self b + anObject b
		c: self c + anObject c
		d: self d + anObject d! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 02:46'!
- anObject
	"Answer the difference between the receiver and the argument."

	^ self + anObject negated! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 02:47'!
/ anObject
	"Answer the division of the receiver by the argument."

	^ self * anObject reciprocal! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 02:47'!
conjugated
	"Answer the conjugate of the receiver."

	^ self class
		a: self a
		b: self b negated
		c: self c negated
		d: self d negated! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 03:29'!
i
	"Answer the receiver multiplicated by i."

	^ self class a: self b negated b: self a c: self d d: self c negated! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 03:29'!
j
	"Answer the receiver multiplicated by j."

	^ self class a: self c negated b: self d negated c: self a d: self b! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 03:30'!
k
	"Answer the receiver multiplicated by k."

	^ self class a: self d negated b: self c c: self b negated d: self a! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 02:46'!
negated
	"Answer the additive inverse of the receiver."

	^ self class
		a: self a negated
		b: self b negated
		c: self c negated
		d: self d negated! !

!Quaternion methodsFor: 'arithmetic' stamp: 'len 2/25/98 02:58'!
reciprocal
	"Answer the multiplicative inverse of the receiver."

	^ self conjugated / self norm2! !


!Quaternion methodsFor: 'mathematical functions' stamp: 'len 2/25/98 02:59'!
norm2
	"Answer the square norm of the receiver."

	^ self a squared + self b squared + self c squared + self d squared! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'len 2/25/98 03:02'!
raisedToInteger: anInteger
	"Answer the receiver raised to the power anInteger
	where the argument must be a kind of Integer."

	anInteger isInteger
		ifFalse: [^ self error: 'raisedToInteger: only works for integral arguments'].
	anInteger = 0 ifTrue: [^ self identity].
	anInteger = 1 ifTrue: [^ self].
	anInteger > 1 ifTrue:
		[^ (self * self raisedToInteger: anInteger // 2)
		 * (self raisedToInteger: anInteger \\ 2)].
	^ (self raisedToInteger: anInteger negated) reciprocal! !

!Quaternion methodsFor: 'mathematical functions' stamp: 'len 2/25/98 03:40'!
squared
	"Answer the square of the receiver."

	^ self * self! !


!Quaternion methodsFor: 'comparing' stamp: 'len 2/25/98 03:04'!
= anObject
	"Answer true if the receiver equals the argument."

	anObject isNumber ifFalse: [^ false].
	anObject isQuaternion
		ifFalse: [^ (anObject adaptQuaternion: self) = anObject adaptToQuaternion].
	^ self a = anObject a
		and: [self b = anObject b
			and: [self c = anObject c
				and: [self d = anObject d]]]! !

!Quaternion methodsFor: 'comparing' stamp: 'len 2/25/98 03:04'!
hash
	"Answer the hash value of the receiver."

	^ self a hash + self b hash + self c hash + self d hash! !


!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:19'!
adaptAlgebraic: anAlgebraicNumber
	"If I am involved in arithmetic with an AlgebraicNumber, do not convert me."
	^ anAlgebraicNumber asQuaternion! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:20'!
adaptComplex: aComplex
	"If I am involved in arithmetic with a Complex, do not convert me."
	^ aComplex asQuaternion! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:19'!
adaptFloat: aFloat
	"If I am involved in arithmetic with a Float, convert the Float."
	^ aFloat asQuaternion! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:20'!
adaptFraction: aFraction
	"If I am involved in arithmetic with a Fraction, convert the Fraction."
	^ aFraction asQuaternion! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:20'!
adaptInteger: anInteger
	"If I am involved in arithmetic with an Integer, convert the Integer."
	^ anInteger asQuaternion! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:19'!
adaptToAlgebraic
	"If I am involved in arithmetic with an AlgebraicNumber, do not convert me."
	^ self! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:20'!
adaptToComplex
	"If I am involved in arithmetic with a Complex, do not convert me."
	^ self! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:19'!
adaptToFloat
	"If I am involved in arithmetic with a Float, do not convert me."
	^ self! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:19'!
adaptToFraction
	"If I am involved in arithmetic with a Fraction, do not convert me."
	^ self! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:19'!
adaptToInteger
	"If I am involved in arithmetic with an Integer, do not convert me."
	^ self! !

!Quaternion methodsFor: 'converting' stamp: 'len 2/25/98 03:21'!
asQuaternion
	^ self! !


!Quaternion methodsFor: 'testing' stamp: 'len 2/25/98 03:05'!
isNull
	"Answer true if the receiver is null."
	^ self a isNull and: [self b isNull and: [self c isNull and: [self d isNull]]]! !

!Quaternion methodsFor: 'testing' stamp: 'len 2/25/98 03:05'!
isNumber
	^ true! !

!Quaternion methodsFor: 'testing' stamp: 'len 2/25/98 03:17'!
isQuaternion
	"Answer true if the receiver is a Quaternion."

	^ true! !


!Quaternion methodsFor: 'constants' stamp: 'len 2/25/98 03:16'!
identity
	"Answer the identity quaternion."

	^ self class a: self a identity b: self b null c: self c null d: self d null! !

!Quaternion methodsFor: 'constants' stamp: 'len 2/25/98 03:16'!
null
	"Answer the null quaternion."

	^ self class a: self a null b: self b null c: self c null d: self d null! !


!Quaternion methodsFor: 'printing' stamp: 'len 2/25/98 03:37'!
printOn: aStream
	"Print a representation of the receiver on the stream aStream."

	| something number |
	self isNull ifTrue: [aStream print: 0. ^ self].
	something _ false.
	self a isNull ifFalse: [aStream print: self a. something _ true].
	self b isNull ifFalse: [number _ self b.
						something
							ifTrue: [aStream nextPutAll: (number positive
								ifTrue: [' + ']
								ifFalse: [number _ number negated. ' - '])]
							ifFalse: [number negative
								ifTrue: [aStream nextPut: $-.
										number _ number negated]].
						number = number identity ifFalse: [aStream print: number; space].
						aStream nextPut: $i. something _ true].
	self c isNull ifFalse: [number _ self c.
						something
							ifTrue: [aStream nextPutAll: (number positive
								ifTrue: [' + ']
								ifFalse: [number _ number negated. ' - '])]
							ifFalse: [number negative
								ifTrue: [aStream nextPut: $-.
										number _ number negated]].
						number = number identity ifFalse: [aStream print: number; space].
						aStream nextPut: $j. something _ true].
	self d isNull ifFalse: [number _ self d.
						something
							ifTrue: [aStream nextPutAll: (number positive
								ifTrue: [' + ']
								ifFalse: [number _ number negated. ' - '])]
							ifFalse: [number negative
								ifTrue: [aStream nextPut: $-.
										number _ number negated]].
						number = number identity ifFalse: [aStream print: number; space].
						aStream nextPut: $k. something _ true]! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Quaternion class
	instanceVariableNames: ''!

!Quaternion class methodsFor: 'instance creation' stamp: 'len 2/25/98 02:42'!
a: aNumber b: bNumber c: cNumber d: dNumber
	"Answer a new instance of the receiver with the given components."

	^ self new a: aNumber b: bNumber c: cNumber d: dNumber! !


!Quaternion class methodsFor: 'examples' stamp: 'len 5/3/98 00:19'!
i
	^ Quaternion a: 0 b: 1 c: 0 d: 0! !

!Quaternion class methodsFor: 'examples' stamp: 'len 5/3/98 00:20'!
j
	^ Quaternion a: 0 b: 0 c: 1 d: 0! !

!Quaternion class methodsFor: 'examples' stamp: 'len 5/3/98 00:20'!
k
	^ Quaternion a: 0 b: 0 c: 0 d: 1! !


Object subclass: #RationalFunction
	instanceVariableNames: 'numerator denominator '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-Kernel'!
!RationalFunction commentStamp: 'A.T. 9/17/1998 02:24' prior: 0!
My instances are rational functions in one or many variables.
Each of my instances has a numerator and a denominator, which are keeped 'reduced' in the univariate case (see RationalFunction>>reduced).
RationalFunctions are created by sending to a Polynomial the message #/ with argument anotherPolynomial.

This is an univariate example:
	Polynomial x + 1 / Polynomial x squared.

And this one is an example with many variables:
	Polynomial x + Polynomial y / Polynomial z squared.
!


!RationalFunction methodsFor: 'accessing'!
denominator
	"Answer the denominator."
	^ denominator! !

!RationalFunction methodsFor: 'accessing' stamp: 'len 4/1/98 23:42'!
indeterminates
	"Answer the indeterminates of the receiver."

	^ Set new
		addAll: self numerator indeterminates;
		addAll: self denominator indeterminates;
		yourself! !

!RationalFunction methodsFor: 'accessing'!
numerator
	"Answer the numerator."
	^ numerator! !


!RationalFunction methodsFor: 'arithmetic' stamp: 'len 4/1/98 23:41'!
* anObject
	"Answer the multiplication of the receiver and the argument."

	| g1 g2 newDenominator newNumerator |
	anObject isRationalFunction
		ifFalse: [^ self numerator * anObject / self denominator].
	(self isUnivariate and: [anObject isUnivariate])
		ifTrue: [g1 _ self numerator gcd: anObject denominator.
				g2 _ self denominator gcd: anObject numerator.
				newNumerator _ self numerator // g1 * (anObject numerator // g2).
				newDenominator _ self denominator // g2 * (anObject denominator // g1).
				^ (newDenominator = newDenominator identity or: [newNumerator isNull])
					ifTrue: [newNumerator]
					ifFalse: [self class numerator: newNumerator denominator: newDenominator]].
	^ self class
		numerator: self numerator * anObject numerator
		denominator: self denominator * anObject denominator! !

!RationalFunction methodsFor: 'arithmetic' stamp: 'len 4/1/98 23:59'!
+ anObject
	"Answer the sum of the receiver and the argument."
	| crossProduct rden1 rden2 denGcd crossGcd newDen |

	anObject isRationalFunction
		ifFalse: [^ self numerator + (self denominator * anObject) / self denominator].
	(self isUnivariate and: [anObject isUnivariate])
		ifTrue: [denGcd _ self denominator gcd: anObject denominator.
				rden1  _ self denominator // denGcd.
				rden2 _ anObject denominator // denGcd.
				crossProduct _ self numerator * rden2 + (anObject numerator * rden1).
				crossGcd _ crossProduct gcd: denGcd.
				((newDen _ denGcd // crossGcd * rden1 * rden2) = 1 or: [crossProduct isNull])
					ifTrue: [^ crossProduct // crossGcd].
				^ self class
					numerator: crossProduct // crossGcd
					denominator: newDen].
	^ (self class
		numerator: self numerator * anObject denominator
					+ (self denominator * anObject numerator)
		denominator: self denominator * anObject denominator) reduced! !

!RationalFunction methodsFor: 'arithmetic' stamp: 'len 8/18/97 19:15'!
- anObject
	"Answer the difference of the receiver and the argument."

	^ self + anObject negated! !

!RationalFunction methodsFor: 'arithmetic' stamp: 'len 8/18/97 19:16'!
/ anObject
	"Answer the division of the receiver by the argument."

	^ self * anObject reciprocal! !

!RationalFunction methodsFor: 'arithmetic'!
negated 
	"Answer the additive inverse of the receiver."

	^ self class
		numerator: numerator negated
		denominator: denominator! !

!RationalFunction methodsFor: 'arithmetic' stamp: 'len 10/18/97 02:55'!
reciprocal 
	"Answer the multiplicative inverse of the receiver."

	| mult |
	mult _ self numerator leadingCoefficient reciprocal.
	self numerator isConstant ifTrue: [^ denominator * mult].
	^ self class numerator: denominator * mult denominator: numerator * mult! !


!RationalFunction methodsFor: 'mathematical functions' stamp: 'len 9/1/97 00:45'!
raisedToInteger: anInteger 
	"Answer the receiver raised to the power anInteger
	where the argument must be a kind of Integer."

	^ self class
		numerator: (self numerator raisedToInteger: anInteger)
		denominator: (self denominator raisedToInteger: anInteger)! !

!RationalFunction methodsFor: 'mathematical functions' stamp: 'len 9/1/97 00:45'!
squared
	"Answer the square of the receiver."

	^ self class
		numerator: self numerator squared
		denominator: self denominator squared! !


!RationalFunction methodsFor: 'operations' stamp: 'len 9/4/97 00:24'!
degree
	"Answer the degree of the receiver."

	^ self numerator degree - self denominator degree! !

!RationalFunction methodsFor: 'operations' stamp: 'len 4/2/98 00:00'!
degreeIn: anInteger
	"Answer the degree of the receiver in the anInteger-th indeterminate."

	^ (self numerator degreeIn: anInteger) - (self denominator degreeIn: anInteger)! !

!RationalFunction methodsFor: 'operations' stamp: 'len 9/3/97 23:49'!
derivative
	"Answer the derivative of the receiver."

	| num gcd |
	num _
		self denominator * self numerator derivative
		- (self numerator * self denominator derivative).
	gcd _ self denominator gcd: num.
	^ self class numerator: num // gcd denominator: (self denominator // gcd) squared * gcd! !

!RationalFunction methodsFor: 'operations' stamp: 'len 9/3/97 23:35'!
derivativeOfOrder: anInteger
	"Answer the anInteger-th derivative of the receiver."

	| answer |
	answer _ self.
	anInteger timesRepeat: [answer _ answer derivative].
	^ answer! !

!RationalFunction methodsFor: 'operations' stamp: 'len 9/4/97 00:14'!
taylorAt: anObject
	"Answer the Taylor development of the receiver at anObject."

	^ (TaylorSequence fromFunction: self at: anObject) ogf! !

!RationalFunction methodsFor: 'operations' stamp: 'len 9/4/97 00:24'!
valueAt: anObject
	"Answer the result of evaluating the receiver at the argument."

	^ (self numerator valueAt: anObject) / (self denominator valueAt: anObject)! !


!RationalFunction methodsFor: 'comparing' stamp: 'len 4/1/98 23:49'!
= anObject
	"Answer true if the receiver equals the argument."

	^ anObject isRationalFunction
		ifTrue: [self numerator * anObject denominator = (self denominator * anObject numerator)]
		ifFalse: [self denominator * anObject = self numerator]! !

!RationalFunction methodsFor: 'comparing' stamp: 'len 4/1/98 23:49'!
hash
	"Answer the hash value for the receiver."

	^ 0! !


!RationalFunction methodsFor: 'converting' stamp: 'len 8/30/97 01:15'!
adaptFraction: aFraction
	"If I am involved in arithmetic with a Fraction, convert the Fraction."
	^ self class
		numerator: (self numerator adaptFraction: aFraction)
		denominator: self denominator identity! !

!RationalFunction methodsFor: 'converting' stamp: 'len 8/30/97 01:15'!
adaptInteger: anInteger
	"If I am involved in arithmetic with an Integer, convert the Integer."
	^ self class
		numerator: (self numerator adaptInteger: anInteger)
		denominator: self denominator identity! !

!RationalFunction methodsFor: 'converting'!
adaptToFraction
	"If I am involved in arithmetic with a Fraction, do not convert me."
	^self! !

!RationalFunction methodsFor: 'converting'!
adaptToInteger
	"If I am involved in arithmetic with an Integer, do not convert me."
	^self! !

!RationalFunction methodsFor: 'converting' stamp: 'len 9/4/97 00:28'!
asPowerSeries
	"Convert the receiver to PowerSeries."

	^ self taylorAt: self nullCoefficient! !

!RationalFunction methodsFor: 'converting' stamp: 'len 9/9/97 00:45'!
asPowerSeriesOLD
	"Convert the receiver to PowerSeries."

	| k f den dependence initial |
	k _ self denominator degree.
	self numerator degree > k ifTrue: [self halt "not yet implemented"].
	initial _ OrderedCollection with: (self valueAt: 0).
	f _ self.
	(k - 1) timesRepeat: [f _ f derivative. initial add: (f valueAt: 0)].
	den _ self denominator / self denominator independentCoefficient negated.
	dependence _ (0 to: k - 1)
		collect: [ :each | den coefficientAt: k - each].
	^ (ConstantRecurrentSequence initial: initial dependence: dependence) ogf! !


!RationalFunction methodsFor: 'testing' stamp: 'len 9/4/97 00:27'!
isNull
	"Answer true if the receiver is null."

	^ self numerator isNull! !

!RationalFunction methodsFor: 'testing'!
isRationalFunction
	"Answer true if the receiver is a rational function."
	^ true! !

!RationalFunction methodsFor: 'testing' stamp: 'len 4/1/98 23:52'!
isUnivariate
	"Answer true if the receiver is an univariate rational function."
	| nindets dindets |
	nindets _ self numerator indeterminates.
	dindets _ self denominator indeterminates.
	(nindets isEmpty or: [nindets size = 1 and: [nindets includes: 1]])
		ifFalse: [^ false].
	(dindets isEmpty or: [dindets size = 1 and: [dindets includes: 1]])
		ifFalse: [^ false].
	^ true! !


!RationalFunction methodsFor: 'basic' stamp: 'len 7/29/97 23:41'!
identity
	^ self numerator identity! !

!RationalFunction methodsFor: 'basic' stamp: 'len 7/29/97 23:40'!
identityCoefficient
	^ self numerator identityCoefficient! !

!RationalFunction methodsFor: 'basic' stamp: 'len 7/29/97 23:41'!
null
	^ self numerator null! !

!RationalFunction methodsFor: 'basic' stamp: 'len 7/29/97 23:40'!
nullCoefficient
	^ self numerator nullCoefficient! !


!RationalFunction methodsFor: 'private' stamp: 'len 4/1/98 23:55'!
reduced
	"Reduce the receiver dividing the numerator and the denominator
	by them GCD and making the denominator monic."
	| gcd mult num den |
	(num _ self numerator) isNull ifTrue: [^ num].
	den _ self denominator.
	self isUnivariate
		ifTrue: [gcd _ num gcd: den.
				num _ num // gcd.
				den _ den // gcd].
	mult _ den asPolynomial leadingCoefficient reciprocal.
	self numerator: num * mult.
	self denominator: den * mult! !


!RationalFunction methodsFor: 'printing' stamp: 'len 8/23/97 02:03'!
printOn: aStream
	"Print a representation of the reseiver on the stream aStream."

	| string |
	aStream print: self numerator; nextPutAll: ' / '.
	string _ self denominator printString.
	((string includes: $+) or: [string includes: $-])
		ifTrue: [aStream nextPut: $(; print: self denominator; nextPut: $)]
		ifFalse: [aStream print: self denominator]! !

!RationalFunction methodsFor: 'printing' stamp: 'len 9/1/97 00:49'!
printOn: aStream withName: aString
	"Print a representation of the receiver on the stream aStream.
	Use the aString to name the indeterminate."

	| string |
	self numerator printOn: aStream withName: aString.
	aStream nextPutAll: ' / '.
	string _ self denominator printString.
	((string includes: $+) or: [string includes: $-])
		ifTrue: [aStream nextPut: $(.
				self denominator printOn: aStream withName: aString.
				aStream nextPut: $)]
		ifFalse: [self denominator printOn: aStream withName: aString]! !

!RationalFunction methodsFor: 'printing' stamp: 'len 4/1/98 23:57'!
printOn: aStream withNames: aDictionary
	"Print a representation of the receiver on the stream aStream.
	Use aDictionary to name the indeterminates."

	| string |
	self numerator printOn: aStream withNames: aDictionary.
	aStream nextPutAll: ' / '.
	string _ self denominator printString.
	((string includes: $+) or: [string includes: $-])
		ifTrue: [aStream nextPut: $(.
				self denominator printOn: aStream withNames: aDictionary.
				aStream nextPut: $)]
		ifFalse: [self denominator printOn: aStream withNames: aDictionary]! !


!RationalFunction methodsFor: 'accessing-private' stamp: 'len 4/1/98 23:53'!
denominator: aPolynomial
	denominator _ aPolynomial! !

!RationalFunction methodsFor: 'accessing-private' stamp: 'len 4/1/98 23:53'!
numerator: aPolynomial
	numerator _ aPolynomial! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RationalFunction class
	instanceVariableNames: ''!

!RationalFunction class methodsFor: 'instance creation' stamp: 'len 4/1/98 23:58'!
numerator: aPolynomial denominator: anotherPolynomial
	"Answer a new instance of the receiver representing a
	rational function with the given numerator and denominator."

	^ self new numerator: aPolynomial; denominator: anotherPolynomial! !
