Object subclass: #NonLinearEqSolver
	instanceVariableNames: 'function '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-NonLinearEqSolver'!

!NonLinearEqSolver methodsFor: 'iteration' stamp: 'A.T. 8/27/1998 19:39'!
iiterate
	self subclassResponsibility! !

!NonLinearEqSolver methodsFor: 'iteration' stamp: 'A.T. 8/27/1998 19:38'!
iterate: anInteger
	"Iterates anInteger times the algorithm"
	anInteger timesRepeat:[self iterate]! !


NonLinearEqSolver subclass: #FixedPointLocator
	instanceVariableNames: 'initial '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-NonLinearEqSolver'!
!FixedPointLocator commentStamp: 'A.T. 9/17/1998 02:24' prior: 0!
I am a representation of a fixed point locator algorithm. Given a Function and an initial value , I start evaluating the function at the initial value, and then I continue evaluating at the result of the last evaluation.

Implementation:
I have two instance variables to represent the function and the initial value (that changes every iteration of myself).

See examples class method.!
]style[(66 8 311)f1,f1LFunction Comment;,f1!


!FixedPointLocator methodsFor: 'locating' stamp: 'A.T. 8/27/1998 18:15'!
iterate
	"Iterates one time the algorithm"
	initial _ function valueAt:initial	! !


!FixedPointLocator methodsFor: 'private' stamp: 'ERG 7/8/1998 17:25'!
function:aFunction
	function _ aFunction! !

!FixedPointLocator methodsFor: 'private' stamp: 'ERG 7/9/1998 10:46'!
initial:aNumber
	initial_aNumber! !

!FixedPointLocator methodsFor: 'private' stamp: 'ERG 7/8/1998 17:25'!
interval:aRealInterval
	interval _ aRealInterval! !


!FixedPointLocator methodsFor: 'accessing' stamp: 'ERG 7/8/1998 17:25'!
function
	^ function! !

!FixedPointLocator methodsFor: 'accessing' stamp: 'ERG 7/9/1998 10:49'!
solution
	"Answer the approximation to the fixed point"
	^ initial! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

FixedPointLocator class
	instanceVariableNames: ''!

!FixedPointLocator class methodsFor: 'examples' stamp: 'ERG 7/9/1998 10:46'!
examples
"
1. f(x) = x^2, initial -0.3
g _ Function from:[:y | y squared].
locator _ FixedPointLocator function:g initial:-0.3.
locator iterate:5.
locator inspect
"! !


!FixedPointLocator class methodsFor: 'instance creation' stamp: 'ERG 7/9/1998 10:44'!
function:aFunction initial:aNumber
	"Answer a new instance of myself with aFunction an aNumber as the input objects"
	^ self new function: aFunction;initial:aNumber! !


NonLinearEqSolver subclass: #RootApproximator
	instanceVariableNames: 'left right iterations '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-NonLinearEqSolver'!

!RootApproximator methodsFor: 'accessing-private' stamp: 'len 5/28/1998 20:58'!
function: anObject left: aNumber right: anotherNumber
	function _ anObject.
	left _ aNumber.
	right _ anotherNumber.
	iterations _ 0! !


!RootApproximator methodsFor: 'accessing' stamp: 'len 5/28/1998 20:57'!
approximation
	"Answer the current approximation computed by the receiver."
	^ left + right / 2! !

!RootApproximator methodsFor: 'accessing' stamp: 'AT 8/25/1998 16:16'!
approximationError
	"Answer the approximation error of the receiver."
	^ right - left / 2! !

!RootApproximator methodsFor: 'accessing' stamp: 'len 5/28/1998 20:54'!
approximationWithError: aNumber
	"Answer an approximate root of the function
	of the receiver with error less than aNumber."
	aNumber strictlyPositive ifFalse: [^ self error: 'approximation error must be strictly positive'].
	[self approximationError < aNumber] whileFalse: [self iterate].
	^ self approximation! !

!RootApproximator methodsFor: 'accessing' stamp: 'len 5/28/1998 20:49'!
function
	^ function! !

!RootApproximator methodsFor: 'accessing' stamp: 'len 5/28/1998 20:55'!
iterate
	"Perform one iteration on the receiver."
	iterations _ iterations + 1! !

!RootApproximator methodsFor: 'accessing' stamp: 'len 5/28/1998 20:55'!
iterations
	"Answer the number of iterations done until now."
	^ iterations! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

RootApproximator class
	instanceVariableNames: ''!

!RootApproximator class methodsFor: 'instance creation' stamp: 'len 5/28/1998 21:01'!
function: anObject left: aNumber right: anotherNumber
	"Answer a new instance of the receiver to approximate a root
	of the function anObject between aNumber and anotherNumber."
	^ self new function: anObject left: aNumber right: anotherNumber! !


RootApproximator subclass: #BisectionRootApproximator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-NonLinearEqSolver'!

!BisectionRootApproximator methodsFor: 'accessing' stamp: 'len 5/28/1998 21:16'!
iterate
	"Perform one iteration on the receiver."
	| mid |
	super iterate.
	mid _ left + right / 2.
	(self function valueAt: mid) sign = (self function valueAt: left) sign
		ifTrue: [left _ mid]
		ifFalse: [right _ mid]! !


RootApproximator subclass: #NewtonRootApproximator
	instanceVariableNames: 'derivative '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-NonLinearEqSolver'!

!NewtonRootApproximator methodsFor: 'accessing-private' stamp: 'len 5/29/1998 00:26'!
derivative: anObject
	derivative _ anObject! !


!NewtonRootApproximator methodsFor: 'accessing' stamp: 'len 5/29/1998 00:21'!
approximation
	"Answer the current approximation computed by the receiver."
	^ left! !

!NewtonRootApproximator methodsFor: 'accessing' stamp: 'len 5/29/1998 00:34'!
approximationError
	"Answer the approximation error of the receiver."
	self notYetImplemented! !

!NewtonRootApproximator methodsFor: 'accessing' stamp: 'len 5/28/1998 21:23'!
derivative
	"Answer the derivative of the function of the receiver."
	derivative isNil ifTrue: [derivative _ self function derivative].
	^ derivative! !

!NewtonRootApproximator methodsFor: 'accessing' stamp: 'len 5/29/1998 00:14'!
iterate
	"Perform one iteration on the receiver."
	super iterate.
	left _ left - ((self function valueAt: left) / (self derivative valueAt: left))! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

NewtonRootApproximator class
	instanceVariableNames: ''!

!NewtonRootApproximator class methodsFor: 'instance creation' stamp: 'len 5/29/1998 00:28'!
function: anObject derivative: anotherObject left: aNumber right: anotherNumber
	"Answer a new instance of the receiver to approximate a root
	of the function anObject, with derivative anotherObject, between
	aNumber and anotherNumber."
	^ (super function: anObject left: aNumber right: anotherNumber)
		derivative: anotherObject! !


RootApproximator subclass: #RegulaFalsiRootApproximator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Math-NonLinearEqSolver'!

!RegulaFalsiRootApproximator methodsFor: 'accessing' stamp: 'len 5/28/1998 21:17'!
iterate
	"Perform one iteration on the receiver."
	| secant fr fl |
	super iterate.
	fr _ self function valueAt: right.
	fl _ self function valueAt: left.
	secant _ right - left * fr / (fl - fr) + right.
	(self function valueAt: secant) sign = fl sign
		ifTrue: [left _ secant]
		ifFalse: [right _ secant]! !
